#! /usr/bin/env python3:

import tempfile
from pterm import aws_config_to_profiles
from pterm import create_aws_profiles
from pterm import sort_aws_config
from pterm import triggers
from pterm import create_k8s_profile
from pterm import find_source_profile
import re
import os
import yaml


def create_config(contents):
    temp = tempfile.NamedTemporaryFile(mode='w', delete=False)
    temp.write(contents)
    return temp.name


def test_aws_config_to_profiles():
    cases = [
        (
            '''
            [profile 1]
            ''',
            {
                '1': {
                    'name': '1',
                    'account': None,
                    'role': None,
                    'azure': False,
                    'source_profile': None,
                },
            },
        ),
        (
            '''
            [profile 2]
            role_arn = arn:partition:service:region:account:resource
            ''',
            {
                '2': {
                    'name': '2',
                    'account': 'account',
                    'role': 'resource',
                    'source_profile': None,
                    'azure': False,
                },
            },
        ),
        (
            '''
            [profile 3]

            [profile 4]
            source_profile = 3
            ''',
            {
                '3': {
                    'name': '3',
                    'account': None,
                    'role': None,
                    'source_profile': None,
                    'azure': False,
                },
                '4': {
                    'name': '4',
                    'account': None,
                    'role': None,
                    'source_profile': '3',
                    'azure': False,
                },
            },
        ),
        (
            '''
            [profile 5]
            azure_tenant_id = foo
            [profile 6]
            source_profile = 5
            ''',
            {
                '5': {
                    'name': '5',
                    'account': None,
                    'role': None,
                    'source_profile': None,
                    'azure': True,
                },
                '6': {
                    'name': '6',
                    'account': None,
                    'role': None,
                    'source_profile': '5',
                    'azure': False,
                },
            },
        ),
    ]

    for case, result in cases:
        config = aws_config_to_profiles(create_config(case))
        assert result == config


def test_create_aws_profiles():
    case = '''
            [profile 1]
           '''
    profiles = create_aws_profiles(create_config(case))
    names = [x['Name'] for x in profiles]
    assert names == list(set(names))

    case = '''
            [profile 2]
            [profile 3]
            source_profile = 2
            '''
    profiles = create_aws_profiles(create_config(case))
    assert profiles[0]['Name'] == '2'
    assert profiles[1]['Name'] == '3'
    assert profiles[2]['Name'] == 'login-2'

    case = '''
            [profile 4]
            azure_tenant_id = foobar
            [profile 5]
            source_profile = 4
            '''
    profiles = create_aws_profiles(create_config(case), azure_path)
    assert profiles[2]['Name'] == 'login-4'
    assert 'aws-azure-login' in profiles[2]['Command']
    assert f'PATH={azure_path()}' in profiles[2]['Command']
    assert 'AWS_PROFILE=4' in profiles[2]['Command']
    assert profiles[1]['Keyboard Map']['0x61-0x80000']['Text'] == 'login-4'


def test_triggers():
    id_rsa_re = [
        x for x in triggers()
        if x['parameter'] == 'id_rsa'
    ][0]['regex']

    cases = [
        # when git push
        f"Enter passphrase for key '{os.getenv('HOME')}/.ssh/id_rsa",
        # when ssh-add
        f"Enter passphrase for {os.getenv('HOME')}/.ssh/id_rsa",
    ]

    for case in cases:
        assert re.search(id_rsa_re, case) is not None


def azure_path():
    return '/some/path'


def test_sort_aws_config():
    case = '''
        [profile b]
        [profile a]
        [profile c]
    '''

    config = create_config(case)
    sort_aws_config(config)

    new_config = [
        x.rstrip() for x in tuple(open(config, 'r')) if x.rstrip()
    ]
    expected = sorted(
        [x.strip() for x in case.split('\n') if x.strip()]
    )
    assert new_config == expected


def test_sort_aws_config_dry():
    case = '''
        [profile b]
        [profile a]
        [profile c]
    '''
    config = create_config(case)
    sort_aws_config(config, dry=True)

    new_config = [
        x.strip() for x in tuple(open(config, 'r')) if x.strip()
    ]
    expected = [x.strip() for x in case.split('\n') if x.strip()]

    assert new_config == expected


def test_create_k8s_profiles():
    case = '''
        [profile aws-profile]
        source_profile = source-profile
        [profile source-profile]
    '''
    aws_profiles = create_aws_profiles(create_config(case), azure_path)

    config = cluster_config('foo')
    profile = create_k8s_profile(config, '/dev/null', aws_profiles)
    assert 'AWS_PROFILE=aws-profile' in profile['Command']

    case = '''
        [profile aws-profile]
        source_profile = source-profile
        [profile source-profile]
    '''
    aws_profiles = create_aws_profiles(create_config(case), azure_path)

    config = cluster_config('foo')
    profile = create_k8s_profile(config, '/dev/null', aws_profiles)
    assert 'AWS_PROFILE=aws-profile' in profile['Command']
    assert 'login-source-profile' == profile['Keyboard Map']['0x61-0x80000']['Text']


def cluster_config(name, aws_profile='aws-profile'):
    return yaml.safe_load(
        'apiVersion: v1\n'
        'clusters:\n'
        '- cluster:\n'
        '    certificate-authority-data: certificate-authority-data\n'
        '    server: server\n'
        f'  name: {name}-name\n'
        'contexts:\n'
        '- context:\n'
        f'    cluster: {name}-name\n'
        f'    user: {name}-user\n'
        f'  name: {name}-name\n'
        f'current-context: {name}-name\n'
        'kind: Config\n'
        'preferences: {}\n'
        'users:\n'
        f'- name: {name}-user\n'
        '  user:\n'
        '    exec:\n'
        '      apiVersion: client.authentication.k8s.io/v1alpha1\n'
        '      args:\n'
        '      - token\n'
        '      - -i\n'
        '      - data\n'
        '      - -r\n'
        '      - arn:aws:iam::account:role\n'
        '      command: aws-iam-authenticator\n'
        '      env:\n'
        '      - name: AWS_PROFILE\n'
        f'        value: {aws_profile}\n'
    )

def test_find_source_profile():
    cases = [
        [
            'profile',
            [{
                'Name': 'profile',
                'Tags': [
                    'source_profile_spf'
                ],
            }],
            'spf',
        ],
        [
            'nonexistingprofile',
            [],
            None,
        ]
    ]

    for profile, profiles, result in cases:
        assert find_source_profile(profile, profiles) == result

    pass




#! /usr/bin/env python3

"""Generate iterm2 profiles for your aws and k8s clusters."""

import configparser
import os
from copy import deepcopy
import shutil
import collections
import difflib
import tempfile

HAS_VAULT = True
try:
    from sh import vault
except ImportError:
    HAS_VAULT = False


def sort_aws_config(path, dry=False):
    """Sort the aws config alphabetically."""
    config = configparser.ConfigParser({}, collections.OrderedDict)
    config.read(path)

    for section in config._sections:
        config._sections[section] = collections.OrderedDict(
            sorted(config._sections[section].items(), key=lambda t: t[0])
        )

    # Order all sections alphabetically
    config._sections = collections.OrderedDict(
        sorted(config._sections.items(), key=lambda t: t[0])
    )

    if dry:
        new_config = tempfile.NamedTemporaryFile()
        with open(new_config.name, 'w') as out:
            config.write(out)
        diff = difflib.unified_diff(
            [x.strip() for x in tuple(open(path, 'r'))],
            [x.strip() for x in tuple(open(new_config.name, 'r'))]
        )
        print('\n'.join(diff))
        return
    with open(path, 'w') as out:
        config.write(out)


def aws_config_to_profiles(aws_config):
    """Convert aws config to iterm2 profiles."""
    config = configparser.ConfigParser()
    config.read(aws_config)
    ret = {}
    for section in config.sections():
        new = {
            'name': section.split()[1],
            'account': None,
            'role': None,
            'azure': config[section].get('azure_tenant_id', None) is not None,
            'source_profile': config[section].get('source_profile', None),
        }
        if 'role_arn' in config[section]:
            new['account'] = config[section]['role_arn'].split(':')[4]
            new['role'] = config[section]['role_arn'].split(':')[5]
        ret[new['name']] = new

    return ret


def aws_azure_login_path():
    """Return the path for aws_azure_login executabl."""
    return which('aws-azure-login')


def which(exe):
    """Return the path for a binary."""
    return os.path.dirname(shutil.which(exe))


def create_aws_profiles(aws_config, azure_path=None):
    """Create aws profiles from a config."""
    if azure_path is None:
        azure_path = aws_azure_login_path
    aws_profiles = aws_config_to_profiles(aws_config)

    profiles = []
    for _, profile in aws_profiles.items():
        new = mkprofile(
            profile['name'],
            source_profile=profile['source_profile'],
            tags=[
                profile['account'],
                profile['role'],
                profile['source_profile']
            ],
        )
        profiles += [new]

    for _, profile in aws_profiles.items():
        source_profile = profile.get("source_profile", None)
        if source_profile is None:
            continue
        new = mkprofile(
            f'login-{source_profile}'
        )
        envs = [f'AWS_PROFILE={source_profile}']
        if aws_profiles[source_profile].get('azure', False):
            path = azure_path()
            envs += [f'PATH={path}']
        node_env = os.getenv('NODE_EXTRA_CA_CERTS', None)
        if node_env is not None:
            envs += [f"NODE_EXTRA_CA_CERTS={node_env}"]
        new["Command"] = f"bash -c '{' '.join(envs)} aws-azure-login --no-prompt || sleep 60'"
        profiles += [new]
    return profiles


def alt_a_split_profile(dictionary, profile):
    """Create the split profile."""
    dictionary['Keyboard Map']["0x61-0x80000"] = {
        "Action": 28,
        "Text": profile,
    }
    return dictionary


def mkprofile(aws_profile, account=None, role=None, source_profile=None, tags=None):
    """Return a new profile."""
    user = os.getenv("USER")
    ret = create_profile(
        aws_profile,
        cmd=f"/usr/bin/env AWS_PROFILE={aws_profile} /usr/bin/login -fp {user}",
        change_title=False,
    )

    if source_profile is not None:
        alt_a_split_profile(ret, f'login-{source_profile}')
        ret['Tags'] += [f'source_profile_{source_profile}']

    if account is not None:
        ret['Tags'] += [account]
    if role is not None:
        ret['Tags'] += [role]
    if tags is not None:
        ret['Tags'] += [x for x in tags if x]

    if 'prod' in aws_profile and 'nonprod' not in aws_profile:
        ret["Background Color"] = {
            "Red Component": 0.217376708984375,
            "Color Space": "sRGB",
            "Blue Component": 0,
            "Alpha Component": 1,
            "Green Component": 0
        }

    return ret


def create_profile(name, cmd=None, change_title=False, tags=None, badge=True):
    """Create a new profile."""
    if tags is None:
        tags = []

    ret = {
        "Name": name,
        "Guid": name,
        "Unlimited Scrollback": True,
        "Title Components": 32,
        "Custom Window Title": name,
        "Allow Title Setting": change_title,
        "Tags": tags,
        "Smart Selection Rules": smart_selection_rules(),
        "Custom Directory": "Recycle",
        "Flashing Bell": True,
        "Silence Bell": True,
        "Triggers": triggers(),
        "Keyboard Map": keybinds(),
    }

    if badge:
        ret["Badge Text"] = name

    if cmd is not None:
        ret["Command"] = cmd
        ret["Custom Command"] = "Yes"
    return ret


def keybinds():
    """Return the dictionary for keybinds."""
    return {
        # cmd + shift + -
        "0x5f-0x120000": {
            "Action": 25,
            "Text": "Split Horizontally with Current Profile\nSplit Horizontally with Current Profile"
        },
        # cmd + shift + \
        "0x7c-0x120000": {
            "Action": 25,
            "Text": "Split Vertically with Current Profile\nSplit Vertically with Current Profile"
        },
    }


def triggers():
    """Return the triggers for profiles."""
    return [
        {
            "partial": True,
            "parameter": "id_rsa",
            "regex": f"^Enter passphrase for (key ')?{os.getenv('HOME')}/.ssh/id_rsa",
            "action": "PasswordTrigger"
        },
        {
            "action": "PasswordTrigger",
            "parameter": "macos",
            "regex": "^Password: .input is hidden.",
            "partial": True
        }
    ]


def smart_selection_rules():
    """Return the smart selection roles for the profiles."""
    return [
        {
            "notes": "terraform aws resource",
            "precision": "normal",
            "regex": "resource \"aws_([a-zA-Z_]*)\"",
            "actions": [
                {
                    "title": "open webpage",
                    "action": 1,
                    "parameter": r"https://www.terraform.io/docs/providers/aws/r/\1.html"
                }
            ]
        },
        {
            "notes": "terraform aws data",
            "precision": "normal",
            "regex": "data \"aws_([a-zA-Z_]*)\"",
            "actions": [
                {
                    "title": "open webpage",
                    "action": 1,
                    "parameter": r"https://www.terraform.io/docs/providers/aws/d/\1.html"
                }
            ]
        },
        {
            "notes": "aws acm-pca",
            "precision": "normal",
            "regex": "arn:aws:acm-pca:([\w-]*):(\d*):certificate-authority/([\w-]*)",
            "actions": [
                {
                    "title": "open webpage",
                    "action": 1,
                    "parameter": r"https://\1.console.aws.amazon.com/acm-pca/home?region=\1#/certificateAuthorities?arn=arn:aws:acm-pca:\1:\2:certificate-authority~2F\3",
                }
            ]
        },
        {
            "notes": "aws iam-policy",
            "precision": "normal",
            "regex": "arn:aws:iam::(\d*):policy/([\w-]*)",
            "actions": [
                {
                    "title": "open webpage",
                    "action": 1,
                    "parameter": r"https://console.aws.amazon.com/iam/home?#/policies/arn:aws:iam::\1:policy/\2$serviceLevelSummary",
                }
            ]
        },
        {
            "notes": "aws iam-role",
            "precision": "normal",
            "regex": "arn:aws:iam::\d*:role/([\w-_]*)",
            "actions": [
                {
                    "title": "open webpage",
                    "action": 1,
                    "parameter": r"https://console.aws.amazon.com/iam/home?#/roles/\1"
                }
            ]
        },
        {
            "notes": "aws lambda",
            "precision": "normal",
            "regex": "arn:aws:lambda:([\w-]*):\d*:function:([\w-_]*)",
            "actions": [
                {
                    "title": "open webpage",
                    "action": 1,
                    "parameter": r"https://\1.console.aws.amazon.com/lambda/home?region=\1#/functions/\2?tab=configuration",
                }
            ]
        },
    ]


def create_k8s_profile(this, cfg, aws_profiles):
    """Create a kubernetes profile."""
    user = os.getenv("USER")
    aws_profile = None
    cluster = this['current-context']
    cmd = [
        "/usr/bin/env",
        f"KUBECONFIG={cfg}",
    ]
    try:
        env = this['users'][0]['user']['exec']['env'][0]
        if env['name'] != 'AWS_PROFILE':
            pass
        aws_profile = env['value']
    except KeyError:
        pass
    except TypeError:
        pass
    except IndexError:
        pass

    if aws_profile is not None:
        cmd += [f'AWS_PROFILE={aws_profile}']

    cmd += [
        '/usr/bin/login',
        '-fp',
        f'{user}',
    ]

    new = create_profile(
        f'k8s-{cluster}',
        cmd=' '.join(cmd),
        change_title=False,
        tags=['k8s'],
    )

    source_profile = find_source_profile(aws_profile, aws_profiles)
    if source_profile is not None:
        new = alt_a_split_profile(new, f'login-{source_profile}')

    new["Background Color"] = {
        "Red Component": 0,
        "Color Space": "sRGB",
        "Blue Component": 0.38311767578125,
        "Alpha Component": 1,
        "Green Component": 0
    }
    return new


def find_source_profile(profile, aws_profiles):
    """Retrieve the source profile for a profile."""
    this_aws_profile = [
        x for x in aws_profiles
        if x.get('Name') == profile
    ]
    try:
        this_aws_profile = this_aws_profile[0]
    except IndexError:
        print(f"Error, profile ${profile} not found in aws config")
        return None

    return [
        x.replace('source_profile_', '')
        for x in this_aws_profile['Tags']
        if x is not None and x.startswith('source_profile_')
    ][0]


def create_vault_profile(name):
    """Create a vault profile."""
    if not HAS_VAULT:
        return {}
    new = create_profile(
        name,
        change_title=False,
        badge=True,
        cmd=f"/bin/bash -c 'PATH={which('vault')} vault server -dev'",
        tags=vault(
            '--version', _env={'VAULT_CLI_NO_COLOR': "true"}
        ).strip().split()
    )
    new['Triggers'] = [
        {
            "action": "CoprocessTrigger",
            "parameter": r"/bin/bash -c 'echo \1 > ~/.vault-token'",
            "regex": r"Root Token: (s\..*)",
            "partial": True
        }
    ]

    new['Keyboard Map'] = {
        "0x77-0x100000": {
            "Action": 12,
            "Text": "Control-w is disabled, please use Control-c to close this tab"
        }
    }

    return new



#! /usr/bin/env python
import setuptools
from pterm import version

setuptools.setup(
    name="pterm",
    version=version.__version__,
    author="Mike Christof",
    author_email="mhristof@gmail.com",
    description="Setup iterm2",
    long_description='\n'.join(tuple(open('README.md', 'r'))),
    long_description_content_type="text/markdown",
    url="https://github.com/mhristof/pterm",
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
    scripts=['scripts/pterm'],
    install_requires=tuple(open('requirements.txt', 'r')),
)
